from fractions import Fraction
from typing import Dict, Final


"""Relic rarity data.

This is a dictionary of dictionaries. The first key is the relic refinement
level. The second key is the relic drop chance for that refinement level.
The value is the chance of getting a common, uncommon, or rare reward from
that relic refinement level.
"""
RELIC_RARITY = {
    "intact": {
        1: Fraction(76, 100 * 3),
        2: Fraction(22, 100 * 2),
        3: Fraction(2, 100),
    },
    "exceptional": {
        1: Fraction(70, 100 * 3),
        2: Fraction(26, 100 * 2),
        3: Fraction(4, 100),
    },
    "flawless": {
        1: Fraction(60, 100 * 3),
        2: Fraction(34, 100 * 2),
        3: Fraction(6, 100),
    },
    "radiant": {
        1: Fraction(50, 100 * 3),
        2: Fraction(40, 100 * 2),
        3: Fraction(10, 100),
    },
}



from relics_run_api.common import RELIC_RARITY
from relics_run_api.history import get_history, get_history_dates
from relics_run_api.index import (
    get_index,
    get_index_price_data,
    get_index_sub_type_data,
)
from relics_run_api.market_data import (
    get_market_data_item_ids,
    get_market_data_item_info,
    get_market_data_items,
)
from relics_run_api.output import (
    get_output_type_dict,
    get_output_wfm_items_categorized,
)



import json
import gzip
import requests
from requests import Response
from typing import Any, Dict, Final

from .base import API_BASE_URL

API_INDEX_URL: Final[str] = API_BASE_URL + "/index"
API_INDEX_INDEX_URL: Final[str] = API_INDEX_URL + "/index.json.gz"
API_INDEX_PRICE_DATA_URL: Final[str] = API_INDEX_URL + "/price_data.json"
API_INDEX_SUB_TYPE_DATA_URL: Final[str] = API_INDEX_URL + "/sub_type_data.json"


def get_index() -> Dict[str, Any]:
    """Get the index from the API.

    :return: The index from the API.
    :rtype: dict
    :raises requests.HTTPError: If there is an error with the request.

    Usage::

        >>> from relics_run_api import get_index
        >>> get_index()
        {
            'relics': {
                'Axi A1': {
                    'Nikana Prime Blueprint': 3,
                    'Akstiletto Prime Barrel': 2,
                    'Dual Kamas Prime Handle': 2,
                    'Trinity Prime Systems Blueprint': 1,
                    'Fragor Prime Head': 1,
                    'Braton Prime Stock': 1
                }, ...
            },
            'non_vaulted': [
                'Axi G10',
                'Lith S15',
                'Neo S17',
                ...
            ],
            'prices': {
                'Frost Prime Set': 100.0,
                'Hydroid Prime Set': 104.5,
                'Zephyr Prime Chassis Blueprint': 5.5,
                ...
            },
            'ducats': {
                'Odonata Prime Blueprint': 45,
                'Odonata Prime Harness Blueprint': 15,
                'Odonata Prime Systems Blueprint': 15,
                ...
            },
            'required_count': {
                'Afuris Prime Barrel': 2,
                'Afuris Prime Receiver': 2,
                'Akbolto Prime Barrel': 2,
                ...
            },
            'types': {
                'Kavasa Prime': 'Skins',
                'Odonata Prime': 'Warframes',
                'Equinox Prime': 'Warframes',
                ...
            }
        }
    """

    # Get the index from the API
    response: Final[Response] = requests.get(API_INDEX_INDEX_URL)

    # Check for errors
    response.raise_for_status()

    # Get the content of the response
    content: Final[str] = response.content

    # Decompress the content
    decompressed_content: Final[str] = gzip.decompress(content)

    # Parse the content
    parsed_content: Final[Dict[str, Any]] = json.loads(decompressed_content)

    # Return the parsed content
    return parsed_content


def get_index_price_data() -> Dict[str, Any]:
    """Get the price data from the API.

    :return: The price data from the API.
    :rtype: dict
    :raises requests.HTTPError: If there is an error with the request.

    Usage::

        >>> from relics_run_api import get_index_price_data
        >>> get_index_price_data()
        {
            "Abating Link": "N/A",
            "Abating Link R0": 14.0,
            "Abating Link R3": 26.666666666666668,
            "Abundant Mutation": "N/A",
            "Abundant Mutation R0": 11.166666666666666,
            "Abundant Mutation R3": 15.5,
            ...
        }
    """
    # Get the price data from the API
    response: Final[Response] = requests.get(API_INDEX_PRICE_DATA_URL)

    # Check for errors
    response.raise_for_status()

    # Return the price data
    return response.json()


def get_index_sub_type_data() -> Dict[str, Any]:
    """Get the sub type data from the API.

    :return: The sub type data from the API.
    :rtype: dict
    :raises requests.HTTPError: If there is an error with the request.

    Usage::

        >>> from relics_run_api import get_index_sub_type_data
        >>> get_index_sub_type_data()
        {
            "Creeping Bullseye": [
                "R0",
                "R5"
            ],
            "Arcane Barrier": [
                "R0",
                "R5"
            ],
            ...
        }
    """
    # Get the sub type data from the API
    response: Final[Response] = requests.get(API_INDEX_SUB_TYPE_DATA_URL)

    # Check for errors
    response.raise_for_status()

    # Return the sub type data
    return response.json()



from datetime import date
import re
import requests
from requests import Response
from typing import Dict, Final, List, Union

from .base import API_BASE_URL

API_HISTORY_URL: Final[str] = API_BASE_URL + "/history"
API_HISTORY_DATE_URL: Final[str] = API_HISTORY_URL + "/price_history_{date}.json"


def get_history_dates() -> List[date]:
    """Get the list of dates for which there is data in the API.

    :return: The list of dates for which there is data in the API.
    :rtype: List[date]
    :raises requests.HTTPError: If there is an error with the request.

    Usage::

        >>> from relics_run_api import get_history_dates
        >>> get_history_dates()
        [datetime.date(2023, 7, 2), datetime.date(2023, 7, 3), datetime.date(2023, 7, 4)]
    """

    # Get the list of dates from the API
    response: Final[Response] = requests.get(API_HISTORY_URL)

    # Check for errors
    response.raise_for_status()

    # Get the content of the response
    content: Final[str] = response.text

    # Create the regex to match the dates
    date_regex = re.compile(r'href="price_history_(\d{4}-\d{2}-\d{2})\.json"')

    # Find all the dates
    dates: List[str] = date_regex.findall(content)

    # Convert the dates to date objects
    dates = [date.fromisoformat(date_string) for date_string in dates]

    # Sort the dates in ascending order
    dates.sort()

    # Return the dates
    return dates


def get_history(date: date) -> Dict[str, List[Dict[str, Union[str, int, float]]]]:
    """Get the data for a given date from the API.

    :param date: The date for which to get the data.
    :type date: date
    :return: The data for the given date.
    :rtype: dict
    :raises requests.HTTPError: If there is an error with the request.

    Usage::

        >>> from datetime import date
        >>> from relics_run_api import get_history
        >>> get_history(date=date(2023, 7, 2))
        {
            'Secura Dual Cestra': [
                {
                    'datetime': '2023-07-02T00:00:00.000+00:00',
                    'volume': 8,
                    'min_price': 25,
                    'max_price': 40,
                    'open_price': 30,
                    'closed_price': 34,
                    'avg_price': 32.5,
                    'wa_price': 32.375,
                    'median': 30.0,
                    'moving_avg': 30.2,
                    'donch_top': 40,
                    'donch_bot': 20,
                    'id': '64a20ff4ca73520023c44a7f'
                }, {
                    'datetime': '2023-07-02T00:00:00.000+00:00',
                    'volume': 435,
                    'min_price': 10,
                    'max_price': 15,
                    'avg_price': 12.5,
                    'wa_price': 10.062,
                    'median': 13,
                    'order_type':
                    'buy',
                    'moving_avg': 21.6,
                    'id': '64a20ff4ca73520023c44a81'
                }, {
                    'datetime': '2023-07-02T00:00:00.000+00:00',
                    'volume': 117,
                    'min_price': 20,
                    'max_price': 45,
                    'avg_price': 32.5,
                    'wa_price': 32.846,
                    'median': 30,
                    'order_type': 'sell',
                    'moving_avg': 25.3,
                    'id': '64a20ff4ca73520023c44a80'
                }
            ],
            [...]
        }
    """

    # Build the URL
    url: Final[str] = API_HISTORY_DATE_URL.format(date=date.isoformat())

    # Get the data from the API
    response: Final[Response] = requests.get(url)

    # Check for errors
    response.raise_for_status()

    # Return the data
    return response.json()



import requests
from typing import Dict, List, Final

from .base import API_BASE_URL

API_MARKET_DATA_URL: Final[str] = API_BASE_URL + "/market_data"
API_MARKET_DATA_ITEM_IDS_URL: Final[str] = API_MARKET_DATA_URL + "/item_ids.json"
API_MARKET_DATA_ITEM_INFO_URL: Final[str] = API_MARKET_DATA_URL + "/item_info.json"
API_MARKET_DATA_ITEMS_URL: Final[str] = API_MARKET_DATA_URL + "/items.json"


def get_market_data_item_ids() -> Dict[str, str]:
    """Get the list of item IDs for which there is data.

    :return: The list of item IDs for which there is data.
    :rtype: Dict[str, str]
    :raises requests.HTTPError: If there is an error with the request.

    Usage::

        >>> from relics_run_api import get_market_data_item_ids
        >>> get_market_data_item_ids()
        {
            "Secura Dual Cestra": "54aae292e7798909064f1575",
            "Creeping Bullseye": "54ca39abe7798915c1c11e10",
            "Mutalist Alad V Assassinate (Key)": "54d4c727e77989281cc7d753",
            ...
        }
    """
    # Get the list of item IDs from the API
    response: Final[Response] = requests.get(API_MARKET_DATA_ITEM_IDS_URL)

    # Check for errors
    response.raise_for_status()

    # Return the list of item IDs
    return response.json()


def get_market_data_item_info() -> Dict[str, dict]:
    """Get the list of item info for which there is data.

    :return: The list of item info for which there is data.
    :rtype: Dict[str, dict]
    :raises requests.HTTPError: If there is an error with the request.

    Usage::

        >>> from relics_run_api import get_market_data_item_info
        >>> get_market_data_item_info()
        {
            "54aae292e7798909064f1575": {
                "set_items": [],
                "item_id": "54aae292e7798909064f1575",
                "tags": [
                    "syndicate",
                    "weapon",
                    "secondary"
                ],
                "mod_max_rank": null,
                "subtypes": []
            },
            ...
        }
    """
    # Get the list of item info from the API
    response: Final[Response] = requests.get(API_MARKET_DATA_ITEM_INFO_URL)

    # Check for errors
    response.raise_for_status()

    # Return the list of item info
    return response.json()


def get_market_data_items() -> List[Dict[str, str]]:
    """Get the list of items for which there is data in the API.

    :return: The list of items for which there is data in the API.
    :rtype: List[Dict[str, str]]
    :raises requests.HTTPError: If there is an error with the request.

    Usage::

        >>> from relics_run_api import get_market_data_items
        >>> get_market_data_items()
        [
            {'name': 'Axi A1'},
            {'name': 'Axi A2'},
            {'name': 'Axi A3'},
            ...
        ]
    """

    # Get the list of items from the API
    response: Final[Response] = requests.get(API_MARKET_DATA_ITEMS_URL)

    # Check for errors
    response.raise_for_status()

    # Return the list of items
    return response.json()



import requests
from typing import Dict, List, Final

from .base import API_BASE_URL

API_OUTPUT_URL: Final[str] = API_BASE_URL + "/output"
API_OUTPUT_TYPE_DICT_URL: Final[str] = API_OUTPUT_URL + "/type_dict.json"
API_OUTPUT_WFM_ITEMS_CATEGORIZED_URL: Final[str] = (
    API_OUTPUT_URL + "/wfm_items_categorized.json"
)


def get_output_type_dict() -> Dict[str, List[str]]:
    """Get the list of types from the API.

    :return: The list of types from the API.
    :rtype: Dict[str, List[str]]
    :raises requests.HTTPError: If there is an error with the request.

    Usage::

        >>> from relics_run_api import get_output_type_dict
        >>> get_output_type_dict()
        {
            "Relics": [
                "Lith K8 Relic",
                "Axi G10 Relic",
                "Lith H1 Relic",
                ...
            ],
            "Arcanes": [
                "Arcane Intention",
                "Arcane Phantasm",
                "Arcane Detoxifier",
                ...
            ],
            ...
        }
    """

    # Get the list of types from the API
    response: Final[Response] = requests.get(API_OUTPUT_TYPE_DICT_URL)

    # Check for errors
    response.raise_for_status()

    # Return the list of types
    return response.json()


def get_output_wfm_items_categorized() -> Dict[str, dict]:
    """Get the list of items from the API.

    :return: The list of items from the API.
    :rtype: Dict[str, dict]
    :raises requests.HTTPError: If there is an error with the request.

    Usage::

        >>> from relics_run_api import get_output_wfm_items_categorized
        >>> get_output_wfm_items_categorized()
        {
            "ArcaneHelmets": {
                "Arcane Aura Helmet": {
                    "item_name": "Arcane Aura Helmet",
                    "url_name": "arcane_aura_trinity_helmet",
                    "thumb": "items/images/en/thumbs/arcane_aura_trinity_helmet.d2b86ccdef9653830055ab389ef0d577.128x128.png",
                    "id": "54aaf1eee779890a8654131d"
                },
                ...
            },
            "Arcanes": {
                "Akimbo Slip Shot": {
                    "item_name": "Akimbo Slip Shot",
                    "url_name": "akimbo_slip_shot",
                    "thumb": "items/images/en/thumbs/akimbo_slip_shot.9cb48cdcc739b5306bb4e2a8b3868e72.128x128.png",
                    "id": "649322b07ec190215a693094"
                },
                ...
            },
            ...
        }
    """

    # Get the list of items from the API
    response: Final[Response] = requests.get(API_OUTPUT_WFM_ITEMS_CATEGORIZED_URL)

    # Check for errors
    response.raise_for_status()

    # Return the list of items
    return response.json()
